package java.io;

import java.util.Locale;
import java.security.AccessController;
import sun.security.action.GetPropertyAction;
import java.nio.charset.UnsupportedCharsetException;
import java.nio.charset.IllegalCharsetNameException;
import java.util.Objects;
import java.nio.charset.Charset;
import java.util.Formatter;

public class PrintWriter extends Writer
{
    protected Writer out;
    private final boolean autoFlush;
    private boolean trouble;
    private Formatter formatter;
    private PrintStream psOut;
    private final String lineSeparator;
    
    private static Charset toCharset(String s) throws UnsupportedEncodingException {
        Objects.requireNonNull(s, "charsetName");
        try {
            return Charset.forName(s);
        }
        catch (IllegalCharsetNameException | UnsupportedCharsetException ex) {
            throw new UnsupportedEncodingException(s);
        }
    }
    
    public PrintWriter(Writer writer) {
        this(writer, false);
    }
    
    public PrintWriter(Writer out, boolean autoFlush) {
        super(out);
        this.trouble = false;
        this.psOut = null;
        this.out = out;
        this.autoFlush = autoFlush;
        this.lineSeparator = (String)AccessController.doPrivileged(new GetPropertyAction("line.separator"));
    }
    
    public PrintWriter(OutputStream outputStream) {
        this(outputStream, false);
    }
    
    public PrintWriter(OutputStream outputStream, boolean b) {
        this(new BufferedWriter(new OutputStreamWriter(outputStream)), b);
        if ((outputStream instanceof PrintStream)) {
            this.psOut = (PrintStream)outputStream;
        }
    }
    
    public PrintWriter(String s) throws FileNotFoundException {
        this(new BufferedWriter(new OutputStreamWriter(new FileOutputStream(s))), false);
    }
    
    private PrintWriter(Charset charset, File file) throws FileNotFoundException {
        this(new BufferedWriter(new OutputStreamWriter(new FileOutputStream(file), charset)), false);
    }
    
    public PrintWriter(String s, String s2) throws FileNotFoundException, UnsupportedEncodingException {
        this(toCharset(s2), new File(s));
    }
    
    public PrintWriter(File file) throws FileNotFoundException {
        this(new BufferedWriter(new OutputStreamWriter(new FileOutputStream(file))), false);
    }
    
    public PrintWriter(File file, String s) throws FileNotFoundException, UnsupportedEncodingException {
        this(toCharset(s), file);
    }
    
    private void ensureOpen() throws IOException {
        if (this.out == null) {
            throw new IOException("Stream closed");
        }
        return;
    }
    
    @Override
    public void flush() {
        try {
            synchronized (this.lock) {
                this.ensureOpen();
                this.out.flush();
            }
        }
        catch (IOException ex) {
            this.trouble = true;
        }
    }
    
    @Override
    public void close() {
        try {
            synchronized (this.lock) {
                if (this.out == null) {
                    return;
                }
                this.out.close();
                this.out = null;
            }
        }
        catch (IOException ex) {
            this.trouble = true;
        }
    }
    
    public boolean checkError() {
        if (this.out != null) {
            this.flush();
        }
        if ((this.out instanceof PrintWriter)) {
            return ((PrintWriter)this.out).checkError();
        }
        if (this.psOut != null) {
            return this.psOut.checkError();
        }
        return this.trouble;
    }
    
    protected void setError() {
        this.trouble = true;
    }
    
    protected void clearError() {
        this.trouble = false;
    }
    
    @Override
    public void write(int n) {
        try {
            synchronized (this.lock) {
                this.ensureOpen();
                this.out.write(n);
            }
        }
        catch (InterruptedIOException ex) {
            Thread.currentThread().interrupt();
        }
        catch (IOException ex2) {
            this.trouble = true;
        }
    }
    
    @Override
    public void write(char[] array, int n, int n2) {
        try {
            synchronized (this.lock) {
                this.ensureOpen();
                this.out.write(array, n, n2);
            }
        }
        catch (InterruptedIOException ex) {
            Thread.currentThread().interrupt();
        }
        catch (IOException ex2) {
            this.trouble = true;
        }
    }
    
    @Override
    public void write(char[] array) {
        this.write(array, 0, array.length);
    }
    
    @Override
    public void write(String s, int n, int n2) {
        try {
            synchronized (this.lock) {
                this.ensureOpen();
                this.out.write(s, n, n2);
            }
        }
        catch (InterruptedIOException ex) {
            Thread.currentThread().interrupt();
        }
        catch (IOException ex2) {
            this.trouble = true;
        }
    }
    
    @Override
    public void write(String s) {
        this.write(s, 0, s.length());
    }
    
    private void newLine() {
        try {
            synchronized (this.lock) {
                this.ensureOpen();
                this.out.write(this.lineSeparator);
                if (this.autoFlush) {
                    this.out.flush();
                }
            }
        }
        catch (InterruptedIOException ex) {
            Thread.currentThread().interrupt();
        }
        catch (IOException ex2) {
            this.trouble = true;
        }
    }
    
    public void print(boolean b) {
        this.write(b ? "true" : "false");
    }
    
    public void print(char c) {
        this.write((int)c);
    }
    
    public void print(int i) {
        this.write(String.valueOf(i));
    }
    
    public void print(long l) {
        this.write(String.valueOf(l));
    }
    
    public void print(float f) {
        this.write(String.valueOf(f));
    }
    
    public void print(double d) {
        this.write(String.valueOf(d));
    }
    
    public void print(char[] array) {
        this.write(array);
    }
    
    public void print(String s) {
        if (s == null) {
            s = "null";
        }
        this.write(s);
    }
    
    public void print(Object obj) {
        this.write(String.valueOf(obj));
    }
    
    public void println() {
        this.newLine();
    }
    
    public void println(boolean b) {
        synchronized (this.lock) {
            this.print(b);
            this.println();
        }
    }
    
    public void println(char c) {
        synchronized (this.lock) {
            this.print(c);
            this.println();
        }
    }
    
    public void println(int n) {
        synchronized (this.lock) {
            this.print(n);
            this.println();
        }
    }
    
    public void println(long n) {
        synchronized (this.lock) {
            this.print(n);
            this.println();
        }
    }
    
    public void println(float n) {
        synchronized (this.lock) {
            this.print(n);
            this.println();
        }
    }
    
    public void println(double n) {
        synchronized (this.lock) {
            this.print(n);
            this.println();
        }
    }
    
    public void println(char[] array) {
        synchronized (this.lock) {
            this.print(array);
            this.println();
        }
    }
    
    public void println(String s) {
        synchronized (this.lock) {
            this.print(s);
            this.println();
        }
    }
    
    public void println(Object obj) {
        String value = String.valueOf(obj);
        synchronized (this.lock) {
            this.print(value);
            this.println();
        }
    }
    
    public PrintWriter printf(String s, Object... array) {
        return this.format(s, array);
    }
    
    public PrintWriter printf(Locale locale, String s, Object... array) {
        return this.format(locale, s, array);
    }
    
    public PrintWriter format(String format, Object... args) {
        try {
            synchronized (this.lock) {
                this.ensureOpen();
                if (this.formatter == null || this.formatter.locale() != Locale.getDefault()) {
                    this.formatter = new Formatter(this);
                }
                this.formatter.format(Locale.getDefault(), format, args);
                if (this.autoFlush) {
                    this.out.flush();
                }
            }
        }
        catch (InterruptedIOException ex) {
            Thread.currentThread().interrupt();
        }
        catch (IOException ex2) {
            this.trouble = true;
        }
        return this;
    }
    
    public PrintWriter format(Locale locale, String format, Object... args) {
        try {
            synchronized (this.lock) {
                this.ensureOpen();
                if (this.formatter == null || this.formatter.locale() != locale) {
                    this.formatter = new Formatter(this, locale);
                }
                this.formatter.format(locale, format, args);
                if (this.autoFlush) {
                    this.out.flush();
                }
            }
        }
        catch (InterruptedIOException ex) {
            Thread.currentThread().interrupt();
        }
        catch (IOException ex2) {
            this.trouble = true;
        }
        return this;
    }
    
    @Override
    public PrintWriter append(CharSequence charSequence) {
        if (charSequence == null) {
            this.write("null");
            return this;
        }
        this.write(charSequence.toString());
        return this;
    }
    
    @Override
    public PrintWriter append(CharSequence charSequence, int n, int n2) {
        charSequence = ((charSequence == null) ? "null" : charSequence);
        this.write(charSequence.subSequence(n, n2).toString());
        return this;
    }
    
    @Override
    public PrintWriter append(char c) {
        this.write((int)c);
        return this;
    }
}
